using System;
using System.Runtime.Remoting;
using System.Collections;
using Chat.Model;
using Chat.Controlador;

namespace Chat.Remoting
{
	public delegate void CrearCanalEventHandler(object sender, ChatEventArgs args);
	public delegate void EsborrarCanalEventHandler(object sender, ChatEventArgs args);
	public delegate void EntrarEventHandler(object sender, ChatEventArgs args);
	public delegate void SortirEventHandler(object sender, ChatEventArgs args);
	public delegate void MissatgeEventHandler(object sender, ChatEventArgs args);
	

	/// <summary>
	/// Descripcin breve de Chat.
	/// </summary>
	public class RemotingChat : MarshalByRefObject
	{
		IDadesServidor dades;
		private static RemotingChat instancia;
		private static bool instanciat = false;

		public event EntrarEventHandler EntrarEvent;
		public event SortirEventHandler SortirEvent;
		public event MissatgeEventHandler MissatgeEvent;
		public event CrearCanalEventHandler CrearCanalEvent;
		public event EsborrarCanalEventHandler EsborrarCanalEvent;

		public RemotingChat()
		{
			IModelFactory modelFactory = new ModelFactory();
			dades = modelFactory.crearDadesServidor();
		}

		public static RemotingChat getInstance() 
		{
			if (!instanciat) 
			{
				instancia = new RemotingChat();
				instanciat = true;
			}
			return instancia;
		}

		public bool entrar(string nick, string nom_canal)
		{
			if (dades.existeixCanal(nom_canal) && !dades.existeixUsuariCanal(nick, nom_canal)) 
			{
				dades.afegirUsuariCanal(nick, nom_canal);
				if (EntrarEvent != null)
				{
					this.EntrarEvent(this, new ChatEventArgs(TipusAccio.Entrar, nick, "", nom_canal));
					return true;
				}
				
			}
			
			return false;
		}

		public bool sortir(string nick, string nom_canal)
		{
			if (dades.existeixCanal(nom_canal) && dades.existeixUsuariCanal(nick, nom_canal)) 
			{
				dades.esborrarUsuariCanal(nick, nom_canal);
				if (SortirEvent != null)
				{
					this.SortirEvent(this, new ChatEventArgs(TipusAccio.Sortir, nick, "", nom_canal));
					return true;
				}
			}
			return false;
		}

		public bool enviarMissatge(string nick, string nom_canal, string missatge)
		{
			if (dades.existeixCanal(nom_canal) && dades.existeixUsuariCanal(nick, nom_canal)) 
			{
				if (EntrarEvent != null)
				{
					this.MissatgeEvent(this, new ChatEventArgs(TipusAccio.NouMissatge, nick, missatge, nom_canal));
					return true;
				}
			}
			return false;
		}

		public bool crearCanal(string nom_canal)
		{
			if (! dades.existeixCanal(nom_canal)) 
			{
				dades.crearCanal(nom_canal);
				if (this.CrearCanalEvent != null)
				{
					this.CrearCanalEvent(this, new ChatEventArgs(TipusAccio.CrearCanal, "", "", nom_canal));
					return true;
				}
			}
			return false;
		}

		public bool esborrarCanal(string nom_canal)
		{
			if (dades.existeixCanal(nom_canal)) 
			{
				dades.esborrarCanal(nom_canal);
				if (this.EsborrarCanalEvent != null)
				{
					this.EsborrarCanalEvent(this, new ChatEventArgs(TipusAccio.EsborrarCanal, "", "", nom_canal));
					return true;
				}
			}
			return false;
		}

		public ArrayList getCanals()
		{
			return dades.getCanalsActuals();
		}

		public ArrayList getUsuarisCanal(string nom_canal)
		{
			if (dades.existeixCanal(nom_canal)) 
			{
				return dades.getUsuarisCanal(nom_canal);
			}
			return null;
		}

	}
}
